@@ -1,6 +1,6 @@
 package HTML::FormWidgets;
 
-# @(#)$Id: FormWidgets.pm 28 2008-03-26 15:33:59Z pjf $
+# @(#)$Id: FormWidgets.pm 37 2008-05-19 22:19:56Z pjf $
 
 use strict;
 use warnings;
@@ -10,49 +10,50 @@ use File::Spec::Functions;
 use HTML::Accessors;
 use Readonly;
 
-use version; our $VERSION = qv( sprintf '0.1.%d', q$Rev: 28 $ =~ /\d+/gmx );
+use version; our $VERSION = qv( sprintf '0.1.%d', q$Rev: 37 $ =~ /\d+/gmx );
 
 Readonly my $NUL   => q();
 Readonly my $TTS   => q( ~ );
 Readonly my %ATTRS =>
-   ( ajaxid      => undef,         ajaxtext    => undef,
-     align       => q(left),       all         => [],
-     assets      => $NUL,          atitle      => 'All',
-     base        => $NUL,          behaviour   => q(classic),
-     button      => $NUL,          checked     => 0,
-     class       => $NUL,          clear       => $NUL,
-     columns     => undef,         container   => undef,
-     ctitle      => 'Current',     current     => [],
-     data        => {},            default     => undef,
-     dropcap     => 0,             edit        => 0,
-     elem        => undef,         evnt_hndlr  => 'checkObj.CheckField',
-     field       => $NUL,          fields      => {},
-     form        => {},            'format'    => undef,
-     fhelp       => $NUL,          header      => undef,
-     height      => undef,         hide        => [],
-     hint_title  => 'Handy Hint',  href        => undef,
-     id          => undef,         id2key      => {},
-     key         => $NUL,          key2id      => {},
-     key2url     => {},            labels      => undef,
-     max_length  => undef,         messages    => undef,
-     name        => $NUL,          nb_symbol   => q(&nbsp;&dagger;),
-     node        => undef,         nowrap      => 0,
-     onblur      => undef,         onchange    => undef,
-     onclick     => undef,         onkeypress  => undef,
-     palign      => undef,         path        => undef,
-     prompt      => $NUL,          fields      => {},
-     pwidth      => 40,            required    => 0,
-     root        => undef,         select      => undef,
-     sep         => q(&nbsp;:&nbsp;),
-     space       => q(&nbsp;) x 3, stepno      => undef,
-     style       => $NUL,          subtype     => undef,
-     swidth      => 1000,          tabstop     => 3,
-     target      => $NUL,          templatedir => undef,
-     text        => $NUL,          tip         => $NUL,
-     tiptype     => q(dagger),     title       => $NUL,
-     type        => undef,         url         => undef,
-     value       => 1,             values      => [],
-     width       => undef, );
+   ( ajaxid       => undef,         ajaxtext     => undef,
+     align        => q(left),       all          => [],
+     assets       => $NUL,          atitle       => 'All',
+     base         => $NUL,          behaviour    => q(classic),
+     button       => $NUL,          checked      => 0,
+     class        => $NUL,          clear        => $NUL,
+     columns      => undef,         container    => undef,
+     content_type => q(application/xhtml+xml),
+     ctitle       => 'Current',     current      => [],
+     data         => {},            default      => undef,
+     dropcap      => 0,             edit         => 0,
+     elem         => undef,         evnt_hndlr   => 'checkObj.CheckField',
+     field        => $NUL,          fields       => {},
+     form         => {},            'format'     => undef,
+     fhelp        => $NUL,          header       => undef,
+     height       => undef,         hide         => [],
+     hint_title   => 'Handy Hint',  href         => undef,
+     id           => undef,         id2key       => {},
+     key          => $NUL,          key2id       => {},
+     key2url      => {},            labels       => undef,
+     max_length   => undef,         messages     => undef,
+     name         => $NUL,          nb_symbol    => q(&nbsp;&dagger;),
+     node         => undef,         nowrap       => 0,
+     onblur       => undef,         onchange     => undef,
+     onclick      => undef,         onkeypress   => undef,
+     palign       => undef,         path         => undef,
+     prompt       => $NUL,          fields       => {},
+     pwidth       => 40,            required     => 0,
+     root         => undef,         select       => undef,
+     sep          => q(&nbsp;:&nbsp;),
+     space        => q(&nbsp;) x 3, stepno       => undef,
+     style        => $NUL,          subtype      => undef,
+     swidth       => 1000,          tabstop      => 3,
+     target       => $NUL,          templatedir  => undef,
+     text         => $NUL,          tip          => $NUL,
+     tiptype      => q(dagger),     title        => $NUL,
+     type         => undef,         url          => undef,
+     value        => 1,             values       => [],
+     width        => undef, );
 
 Readonly my @STATIC => (
    qw(atitle align behaviour checked class clear container ctitle edit
@@ -100,13 +101,14 @@ sub build {
 sub new {
    my ($me, @rest) = @_;
    my $args        = $me->_arg_list( @rest );
-   my ($class, $method, $msg_id, $self, $text, @tmp, $val);
+   my ($class, $method, $msg_id, $ref, $self, $text, @tmp, $val);
 
    # Start with some hard coded defaults;
    $self = { %ATTRS };
 
    # Now we can create HTML elements like we could with CGI.pm
-   $self->{elem} = HTML::Accessors->new();
+   $ref = { content_type => $args->{content_type} } if ($args->{content_type});
+   $self->{elem} = HTML::Accessors->new( $ref );
 
    # Bare minimum is fields + id to get a useful widget
    for (qw(ajaxid fields id name)) {
@@ -294,7 +296,9 @@ sub render {
 sub _arg_list {
    my ($me, @rest) = @_;
 
-   return $rest[0] && ref $rest[0] eq q(HASH) ? $rest[0] : { @rest };
+   return {} unless ($rest[0]);
+
+   return ref $rest[0] eq q(HASH) ? $rest[0] : { @rest };
 }
 
 sub _group_fields {
@@ -336,7 +340,7 @@ HTML::FormWidgets - Create HTML form markup
 
 =head1 Version
 
-0.1.$Rev: 28 $
+0.1.$Rev: 37 $
 
 =head1 Synopsis
 
@@ -351,16 +355,17 @@ HTML::FormWidgets - Create HTML form markup
       my $form     = [ $s->{iFrame} ];
       my $config   = {};
 
-      $config->{root       } = $c->config->{root};
-      $config->{base       } = $c->req->base;
-      $config->{url        } = $c->req->path;
-      $config->{assets     } = $s->{assets};
-      $config->{fields     } = $s->{fields} || {};
-      $config->{form       } = $s->{form};
-      $config->{hide       } = $s->{iFrame}->{hidden};
-      $config->{messages   } = $s->{messages};
-      $config->{swidth     } = $s->{width} if ($s->{width});
-      $config->{templatedir} = $c->config->{dynamic_templates};
+      $config->{root        } = $c->config->{root};
+      $config->{base        } = $c->req->base;
+      $config->{content_type} = $c->config->{content_type};
+      $config->{url         } = $c->req->path;
+      $config->{assets      } = $s->{assets};
+      $config->{fields      } = $s->{fields} || {};
+      $config->{form        } = $s->{form};
+      $config->{hide        } = $s->{iFrame}->{hidden};
+      $config->{messages    } = $s->{messages};
+      $config->{swidth      } = $s->{width} if ($s->{width});
+      $config->{templatedir } = $c->config->{dynamic_templates};
 
       HTML::FormWidgets->build( $config, $form );
       return;
@@ -369,13 +374,13 @@ HTML::FormWidgets - Create HTML form markup
 =head1 Description
 
 Transforms a Perl data structure which defines one or more "widgets"
-into XHTML. Each widget is comprised of these optional components: a
-line or question number, a prompt string, a separator, an input field,
-additional field help, and Ajax field error string.
+into HTML or XHTML. Each widget is comprised of these optional
+components: a line or question number, a prompt string, a separator,
+an input field, additional field help, and Ajax field error string.
 
 Input fields are selected by the widget C<type> attribute. A factory
-subclass implements the method that generates the XHTML for that input
-field type. Adding more widget types is straightforward
+subclass implements the method that generates the HTML or XHTML for
+that input field type. Adding more widget types is straightforward
 
 This module is using the MooTools Javascript library
 L<http://mootools.net/> to modify default browser behaviour
@@ -491,55 +496,58 @@ reflect this modules primary use within a L<Catalyst> application):
 
 =over 3
 
-=item C<$c-E<gt>config-E<gt>{root}>
-
-The path to the document root for this application (C<$config-E<gt>root>)
-
-=item C<$c-E<gt>config-E<gt>{dynamic_templates}>
-
-The path to template files used by the C<::Template> subclass
-(C<$config-E<gt>templatedir>)
-
-=item C<$c-E<gt>req-E<gt>base>
-
-This is the prefix for our URI (C<$config-E<gt>base>)
-
-=item C<$c-E<gt>req-E<gt>path>
-
-Only used by the C<::Tree> subclass to create self referential URIs
-(C<$config-E<gt>url>)
-
-=item C<$c-E<gt>stash-E<gt>{assets}>
+=item B<assets>
 
 Some of the widgets require image files. This attribute is used to
-create the URI for those images (C<$config-E<gt>assets>)
+create the URI for those images
 
-=item C<$c-E<gt>stash-E<gt>{fields}>
+=item B<base>
+
+This is the prefix for our URI
+
+=item B<content_type>
+
+Either I<application/xhtml+xml> which generates XHTML 1.1 and is the
+default or I<text/html> which generates HTML 4.01
+
+=item B<fields>
 
 This hash ref contains the fields definitions. Static parameters for
 each widget can be stored in configuration files. This reduces the
 number of attributes that have to be passed in the call to the
 constructor
 
-=item C<$c-E<gt>stash-E<gt>{form}>
+=item B<form>
 
-Used by the C<::Chooser> subclass (C<$config-E<gt>form>)
+Used by the C<::Chooser> subclass
 
-=item C<$c-E<gt>stash-E<gt>{iFrame}-E<gt>{hidden}>
+=item B<hide>
 
 So that the C<::File> and C<::Table> subclasses can store the number
-of rows added as the hidden form variable B<nRows> (C<$config-E<gt>hide>)
+of rows added as the hidden form variable B<nRows>
 
-=item C<$c-E<gt>stash-E<gt>{messages}>
+=item B<messages>
 
 Many of the subclasses use this hash to supply literal text in a
 language of the users choosing
 
-=item C<$c-E<gt>stash-E<gt>{width}>
+=item B<root>
+
+The path to the document root for this application
+
+=item B<swidth>
 
 Width in pixels of the browser window. This is used to calculate the
 width of the field prompt. The field prompt needs to be a fixed length
-so that the separator colons align vertically (C<$config-E<gt>swidth>)
+so that the separator colons align vertically
+
+=item B<templatedir>
+
+The path to template files used by the C<::Template> subclass
+
+=item B<url>
+
+Only used by the C<::Tree> subclass to create self referential URIs
 
 =back
 
@@ -573,6 +581,10 @@ setting
 Creates a popup window which allows one item to be selected from a
 B<long> list of items
 
+=head2 Cloud
+
+Creates list of links from the data set supplied in C<$me-E<gt>data>
+
 =head2 Date
 
 Return another text field, this time with a calendar icon which when
